@@ -45,15 +45,13 @@
 #include "gsd-keyboard-manager.h"
 #include "gsd-input-helper.h"
 #include "gsd-enums.h"
+#include "gsd-settings-migrate.h"
 
 #define GSD_KEYBOARD_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GSD_TYPE_KEYBOARD_MANAGER, GsdKeyboardManagerPrivate))
 
 #define GSD_KEYBOARD_DIR "org.gnome.settings-daemon.peripherals.keyboard"
 
-#define KEY_REPEAT         "repeat"
 #define KEY_CLICK          "click"
-#define KEY_INTERVAL       "repeat-interval"
-#define KEY_DELAY          "delay"
 #define KEY_CLICK_VOLUME   "click-volume"
 #define KEY_REMEMBER_NUMLOCK_STATE "remember-numlock-state"
 #define KEY_NUMLOCK_STATE  "numlock-state"
@@ -153,15 +151,6 @@ schema_is_installed (const char *schema)
                 contained (relocatable, schema));
 }
 
-static gboolean
-xkb_set_keyboard_autorepeat_rate (guint delay, guint interval)
-{
-        return XkbSetAutoRepeatRate (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
-                                     XkbUseCoreKbd,
-                                     delay,
-                                     interval);
-}
-
 static gboolean
 check_xkb_extension (GsdKeyboardManager *manager)
 {
@@ -339,43 +328,9 @@ apply_numlock (GsdKeyboardManager *manager)
         gdk_error_trap_pop_ignored ();
 }
 
-static void
-apply_repeat (GsdKeyboardManager *manager)
-{
-	GSettings       *settings;
-        gboolean         repeat;
-        guint            interval;
-        guint            delay;
-
-        g_debug ("Applying the repeat settings");
-        settings      = manager->priv->settings;
-        repeat        = g_settings_get_boolean  (settings, KEY_REPEAT);
-        interval      = g_settings_get_uint  (settings, KEY_INTERVAL);
-        delay         = g_settings_get_uint  (settings, KEY_DELAY);
-
-        gdk_error_trap_push ();
-        if (repeat) {
-                gboolean rate_set = FALSE;
-
-                XAutoRepeatOn (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()));
-                /* Use XKB in preference */
-                rate_set = xkb_set_keyboard_autorepeat_rate (delay, interval);
-
-                if (!rate_set)
-                        g_warning ("Neither XKeyboard not Xfree86's keyboard extensions are available,\n"
-                                   "no way to support keyboard autorepeat rate settings");
-        } else {
-                XAutoRepeatOff (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()));
-        }
-
-        XSync (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), FALSE);
-        gdk_error_trap_pop_ignored ();
-}
-
 static void
 apply_all_settings (GsdKeyboardManager *manager)
 {
-	apply_repeat (manager);
 	apply_bell (manager);
 	apply_numlock (manager);
 }
@@ -397,11 +352,6 @@ settings_changed (GSettings          *settings,
 		apply_numlock (manager);
 	} else if (g_strcmp0 (key, KEY_NUMLOCK_STATE) == 0) {
 		g_debug ("Num-Lock state '%s' changed, will apply at next startup", key);
-	} else if (g_strcmp0 (key, KEY_REPEAT) == 0 ||
-		 g_strcmp0 (key, KEY_INTERVAL) == 0 ||
-		 g_strcmp0 (key, KEY_DELAY) == 0) {
-		g_debug ("Key repeat setting '%s' changed, applying key repeat settings", key);
-		apply_repeat (manager);
 	} else if (g_strcmp0 (key, KEY_BELL_CUSTOM_FILE) == 0){
 		g_debug ("Ignoring '%s' setting change", KEY_BELL_CUSTOM_FILE);
 	} else {
@@ -443,6 +393,9 @@ set_devicepresence_handler (GsdKeyboardManager *manager)
 {
         GdkDeviceManager *device_manager;
 
+        if (gnome_settings_is_wayland ())
+                return;
+
         device_manager = gdk_display_get_device_manager (gdk_display_get_default ());
 
         manager->priv->device_added_id = g_signal_connect (G_OBJECT (device_manager), "device-added",
@@ -785,12 +738,14 @@ start_keyboard_idle_cb (GsdKeyboardManager *manager)
                                   localed_proxy_ready,
                                   manager);
 
-        /* apply current settings before we install the callback */
-        g_debug ("Started the keyboard plugin, applying all settings");
-        apply_all_settings (manager);
+        if (!gnome_settings_is_wayland ()) {
+                /* apply current settings before we install the callback */
+                g_debug ("Started the keyboard plugin, applying all settings");
+                apply_all_settings (manager);
 
-        g_signal_connect (G_OBJECT (manager->priv->settings), "changed",
-                          G_CALLBACK (settings_changed), manager);
+                g_signal_connect (G_OBJECT (manager->priv->settings), "changed",
+                                  G_CALLBACK (settings_changed), manager);
+        }
 
 	install_xkb_filter (manager);
 
@@ -879,12 +834,29 @@ gsd_keyboard_manager_finalize (GObject *object)
         G_OBJECT_CLASS (gsd_keyboard_manager_parent_class)->finalize (object);
 }
 
+static void
+migrate_keyboard_settings (void)
+{
+        GsdSettingsMigrateEntry entries[] = {
+                { "repeat",          "repeat",          NULL },
+                { "repeat-interval", "repeat-interval", NULL },
+                { "delay",           "delay",           NULL }
+        };
+
+        gsd_settings_migrate_check ("org.gnome.settings-daemon.peripherals.keyboard.deprecated",
+                                    "/org/gnome/settings-daemon/peripherals/keyboard/",
+                                    "org.gnome.desktop.peripherals.keyboard",
+                                    "/org/gnome/desktop/peripherals/keyboard/",
+                                    entries, G_N_ELEMENTS (entries));
+}
+
 GsdKeyboardManager *
 gsd_keyboard_manager_new (void)
 {
         if (manager_object != NULL) {
                 g_object_ref (manager_object);
         } else {
+                migrate_keyboard_settings ();
                 manager_object = g_object_new (GSD_TYPE_KEYBOARD_MANAGER, NULL);
                 g_object_add_weak_pointer (manager_object,
                                            (gpointer *) &manager_object);
